==> haystack-0.4.1.tar.gz: test/ZincTest.dart <==
library haystack.zinctest;

import 'package:unittest/unittest.dart';
import 'package:haystack/core.dart';

main() {
  test('', () {
    verifyGrid(
        "ver:\"2.0\"\n" +
        "fooBar33\n" +
        "\n",
        null,
        [
          "fooBar33",
          null
        ],
        new List<List<HVal>>()
    );
  });

  test('', () {
    verifyGrid(
        "ver:\"2.0\" tag foo:\"bar\"\n" +
        "xyz\n" +
        "\"val\"\n" +
        "\n",
        new HDictBuilder().add("tag", HMarker.VAL).add("foo", new HStr("bar")).toDict(),
        ["xyz", null],
        [[ new HStr("val") ]]
    );
  });

  test('', () {
    verifyGrid(
        "ver:\"2.0\"\n" +
        "val\n" +
        "N\n" +
        "\n",
        null,
        ["val", null],
        [[null]]
    );
  });

  test('', () {
    verifyGrid(
        "ver:\"2.0\"\n" +
        "a,b\n" +
        "1,2\n" +
        "3,4\n" +
        "\n",
        null,
        [
          "a", null,
          "b", null
        ],
        [
          [new HNum(1.0), new HNum(2.0)],
          [new HNum(3.0), new HNum(4.0)]
        ]
    );
  });

  test('', () {
    verifyGrid(
        "ver:\"2.0\" bg: Bin(image/jpeg) mark\n" +
        "file1 dis:\"F1\" icon: Bin(image/gif),file2 icon: Bin(image/jpg)\n" +
        "Bin(text/plain),N\n" +
        "4,Bin(image/png)\n" +
        "Bin(text/html; a=foo; bar=\"sep\"),Bin(text/html; charset=utf8)\n",
        new HDictBuilder().add("bg", new HBin("image/jpeg")).add("mark", HMarker.VAL).toDict(),
        [
          "file1", new HDictBuilder().add("icon", new HBin("image/gif")).add("dis", new HStr("F1")).toDict(),
          "file2", new HDictBuilder().add("icon", new HBin("image/jpg")).toDict()
        ],
        [
          [ new HBin("text/plain"), null ],
          [ new HNum(4.0), new HBin("image/png") ],
          [ new HBin("text/html; a=foo; bar=\"sep\""), new HBin("text/html; charset=utf8") ]
        ]
    );
  });

  test('', () {
    verifyGrid(
        "ver:\"2.0\"\n" +
        "a,    b,      c,      d\n" +
        "T,    F,      N,   -99\n" +
        "2.3,  -5e-10, 2.4e20, 123e-10\n" +
        "\"\",   \"a\",   \"\\\" \\\\ \\t \\n \\r\", \"\\uabcd\"\n" +
        "`path`, @12cbb082-0c02ae73, 4s, -2.5min\n" +
        "M,R,Bin(image/png),Bin(image/png)\n" +
        "2009-12-31, 23:59:01, 01:02:03.123, 2009-02-03T04:05:06Z\n" +
        "INF, -INF, \"\", NaN\n" +
        "C(12,-34),C(0.123,-.789),C(84.5,-77.45),C(-90,180)\n" +
        "\n",
        null,
        [
          "a", null,
          "b", null,
          "c", null,
          "d", null
        ],
        [
          [ HBool.TRUE, HBool.FALSE, null, new HNum(-99.0) ],
          [ new HNum(2.3), new HNum(-5.0E-10), new HNum(2.4E20), new HNum(1.23E-8) ],
          [ new HStr(""), new HStr("a"), new HStr("\" \\ \t \n \r"), new HStr("\uabcd") ],
          [ new HUri("path"), new HRef("12cbb082-0c02ae73", null), new HNum(4.0, "s"), new HNum(-2.5, "min") ],
          [ HMarker.VAL, new HStr("_remove_"), new HBin("image/png"), new HBin("image/png") ],
          [ new HDate(2009, 12, 31), new HTime(23, 59, 1, 0), new HTime(1, 2, 3, 123), new HDateTime(new HDate(2009, 2, 3),new HTime(4, 5, 6, 0),new HTimeZone("UTC")) ],
          [ HNum.POS_INF, HNum.NEG_INF, new HStr(""), HNum.NaN ],
          [ new HCoord(12.0, -34.0), new HCoord(0.123, -0.789), new HCoord(84.5, -77.45), new HCoord(-90.0, 180.0) ]
        ]
    );
  });

  test('', () {
    verifyGrid(
        "ver:\"2.0\"\n" +
        "foo\n" +
        "`foo\$20bar`\n" +
        "`foo\\`bar`\n" +
        "`file \\#2`\n" +
        "\"\$15\"\n",
        null,
        [
          "foo", null,
        ],
        [
          [ new HUri("foo\$20bar") ],
          [ new HUri("foo`bar") ],
          [ new HUri("file \\#2") ],
          [ new HStr("\$15") ]
        ]
    );
  });

  test('', () {
    verifyGrid(
        "ver:\"2.0\"\n" +
        "a, b\n" +
        "-3.1kg,4kg\n" +
        "5%,3.2%\n" +
        "5kWh/ft\u00b2,-15kWh/m\u00b2\n" +
        "123e+12kJ/kg_dry,74\u0394\u00b0F\n",
        null,
        [
          "a", null,
          "b", null
        ],
        [
          [ new HNum(-3.1, "kg"), new HNum(4.0, "kg") ],
          [ new HNum(5.0, "%"), new HNum(3.2, "%") ],
          [ new HNum(5.0, "kWh/ft\u00b2"), new HNum(-15.0, "kWh/m\u00b2") ],
          [ new HNum(1.23E14, "kJ/kg_dry"), new HNum(74.0, "\u0394\u00b0F") ],
        ]
    );
  });

  test('', () {
    verifyGrid(
        "ver:\"2.0\"\n" +
        "a, b, c\n" +
        ", 1, 2\n" +
        "3, , 5\n" +
        "6, 7_000,\n" +
        ",,10\n" +
        ",,\n" +
        "14,,\n" +
        "\n",
        null,
        [
          "a", null,
          "b", null,
          "c", null,
        ],
        [
          [ null, new HNum(1.0), new HNum(2.0) ],
          [ new HNum(3.0), null, new HNum(5.0) ],
          [ new HNum(6.0), new HNum(7000.0), null ],
          [ null, null, new HNum(10.0) ],
          [ null, null, null ],
          [ new HNum(14.0), null, null ],
        ]
    );
  });

  test('', () {
    verifyGrid(
        "ver:\"2.0\"\n" +
        "a,b\n" +
        "2010-03-01T23:55:00.013-05:00 GMT+5,2010-03-01T23:55:00.013+10:00 GMT-10\n",
        null,
        [
          "a", null,
          "b", null,
        ],
        [
          [ new HDateTime(new HDate(2010, 3, 1),new HTime(23, 55, 0, 13),new HTimeZone("GMT+5", -18000)), new HDateTime(new HDate(2010, 3, 1),new HTime(23, 55, 0, 13),new HTimeZone("GMT-10", 36000)) ]
        ]
    );
  });

  test('', () {
    verifyGrid(
        "ver:\"2.0\" a: 2009-02-03T04:05:06Z foo b: 2010-02-03T04:05:06Z UTC bar c: 2009-12-03T04:05:06Z London baz\n" +
        "a\n" +
        "3.814697265625E-6\n" +
        "2010-12-18T14:11:30.924Z\n" +
        "2010-12-18T14:11:30.925Z UTC\n" +
        "2010-12-18T14:11:30.925Z London\n" +
        "45\$\n" +
        "33\u00a3\n" +
        "@12cbb08e-0c02ae73\n" +
        "7.15625E-4kWh/ft\u00b2\n",
        new HDictBuilder().add("b", new HDateTime(new HDate(2010, 2, 3),new HTime(4, 5, 6, 0),new HTimeZone("UTC"))).add("baz", HMarker.VAL).add("c", new HDateTime(new HDate(2009, 12, 3),new HTime(4, 5, 6, 0),new HTimeZone("London"))).add("a", new HDateTime(new HDate(2009, 2, 3),new HTime(4, 5, 6, 0),new HTimeZone("UTC"))).add("foo", HMarker.VAL).add("bar", HMarker.VAL).toDict(),
        [
          "a", null,
        ],
        [
          [ new HNum(3.814697265625E-6) ],
          [ new HDateTime(new HDate(2010, 12, 18),new HTime(14, 11, 30, 924),new HTimeZone("UTC")) ],
          [ new HDateTime(new HDate(2010, 12, 18),new HTime(14, 11, 30, 925),new HTimeZone("UTC")) ],
          [ new HDateTime(new HDate(2010, 12, 18),new HTime(14, 11, 30, 925),new HTimeZone("London")) ],
          [ new HNum(45.0, "\$") ],
          [ new HNum(33.0, "\u00a3") ],
          [ new HRef("12cbb08e-0c02ae73", null) ],
          [ new HNum(7.15625E-4, "kWh/ft\u00b2") ],
        ]
    );
  });

}

void verifyGrid(String str, HDict meta, List<Object> cols, List<List<HVal>> rows) {
/*
    System.out.println();
    System.out.println("###############################################");
    System.out.println();
    System.out.println(str);
    */

// normalize nulls
  if (meta == null) meta = HDict.EMPTY;
  for (int i=0; i<cols.length; ++i)
    if (cols[i] == null) cols[i] = HDict.EMPTY;

// read from zinc
  HGrid grid = new HZincReader(str).readGrid();
  verifyGridEq(grid, meta, cols, rows);

// write grid and verify we can parse that too
  String writeStr = HZincWriter.gridToString(grid);
  HGrid writeGrid = new HZincReader(writeStr).readGrid();
  verifyGridEq(writeGrid, meta, cols, rows);
}

void verifyGridEq(HGrid grid, HDict meta, List<Object> cols, List<List<HVal>> rows) {
// meta
  expect(grid.meta, equals(meta));

// cols
  expect(grid.numCols, equals(cols.length/2));
  for (int i=0; i<grid.numCols; ++i) {
    expect(grid.colByIndex(i).name, equals(cols[i*2+0]));
    expect(grid.colByIndex(i).meta, equals(cols[i*2+1]));
  }

// rows
  expect(grid.numRows, equals(rows.length));
  for (int ri=0; ri<rows.length; ++ri) {
    List<HVal> expected = rows[ri];
    HRow actual = grid.row(ri);
    for (int ci=0; ci<expected.length; ++ci) {
      expect(expected[ci], equals(actual.get(grid.colByIndex(ci).name, checked:false)));
    }
  }
}

==> haystack-0.4.11.tar.gz: test/ZincTest.dart <==
library haystack.zinctest;

import 'package:unittest/unittest.dart';
import 'package:haystack/core.dart';

main() {
  test('', () {
    verifyGrid(
        "ver:\"2.0\"\n" +
        "fooBar33\n" +
        "\n",
        null,
        [
          "fooBar33",
          null
        ],
        new List<List<HVal>>()
    );
  });

  test('', () {
    verifyGrid(
        "ver:\"2.0\" tag foo:\"bar\"\n" +
        "xyz\n" +
        "\"val\"\n" +
        "\n",
        new HDictBuilder().add("tag", HMarker.VAL).add("foo", new HStr("bar")).toDict(),
        ["xyz", null],
        [[ new HStr("val") ]]
    );
  });

  test('', () {
    verifyGrid(
        "ver:\"2.0\"\n" +
        "val\n" +
        "N\n" +
        "\n",
        null,
        ["val", null],
        [[null]]
    );
  });

  test('', () {
    verifyGrid(
        "ver:\"2.0\"\n" +
        "a,b\n" +
        "1,2\n" +
        "3,4\n" +
        "\n",
        null,
        [
          "a", null,
          "b", null
        ],
        [
          [new HNum(1.0), new HNum(2.0)],
          [new HNum(3.0), new HNum(4.0)]
        ]
    );
  });

  test('', () {
    verifyGrid(
        "ver:\"2.0\" bg: Bin(image/jpeg) mark\n" +
        "file1 dis:\"F1\" icon: Bin(image/gif),file2 icon: Bin(image/jpg)\n" +
        "Bin(text/plain),N\n" +
        "4,Bin(image/png)\n" +
        "Bin(text/html; a=foo; bar=\"sep\"),Bin(text/html; charset=utf8)\n",
        new HDictBuilder().add("bg", new HBin("image/jpeg")).add("mark", HMarker.VAL).toDict(),
        [
          "file1", new HDictBuilder().add("icon", new HBin("image/gif")).add("dis", new HStr("F1")).toDict(),
          "file2", new HDictBuilder().add("icon", new HBin("image/jpg")).toDict()
        ],
        [
          [ new HBin("text/plain"), null ],
          [ new HNum(4.0), new HBin("image/png") ],
          [ new HBin("text/html; a=foo; bar=\"sep\""), new HBin("text/html; charset=utf8") ]
        ]
    );
  });

  test('', () {
    verifyGrid(
        "ver:\"2.0\"\n" +
        "a,    b,      c,      d\n" +
        "T,    F,      N,   -99\n" +
        "2.3,  -5e-10, 2.4e20, 123e-10\n" +
        "\"\",   \"a\",   \"\\\" \\\\ \\t \\n \\r\", \"\\uabcd\"\n" +
        "`path`, @12cbb082-0c02ae73, 4s, -2.5min\n" +
        "M,R,Bin(image/png),Bin(image/png)\n" +
        "2009-12-31, 23:59:01, 01:02:03.123, 2009-02-03T04:05:06Z\n" +
        "INF, -INF, \"\", NaN\n" +
        "C(12,-34),C(0.123,-.789),C(84.5,-77.45),C(-90,180)\n" +
        "\n",
        null,
        [
          "a", null,
          "b", null,
          "c", null,
          "d", null
        ],
        [
          [ HBool.TRUE, HBool.FALSE, null, new HNum(-99.0) ],
          [ new HNum(2.3), new HNum(-5.0E-10), new HNum(2.4E20), new HNum(1.23E-8) ],
          [ new HStr(""), new HStr("a"), new HStr("\" \\ \t \n \r"), new HStr("\uabcd") ],
          [ new HUri("path"), new HRef("12cbb082-0c02ae73", null), new HNum(4.0, "s"), new HNum(-2.5, "min") ],
          [ HMarker.VAL, HRemove.VAL, new HBin("image/png"), new HBin("image/png") ],
          [ new HDate(2009, 12, 31), new HTime(23, 59, 1, 0), new HTime(1, 2, 3, 123), new HDateTime(new HDate(2009, 2, 3),new HTime(4, 5, 6, 0),new HTimeZone("UTC")) ],
          [ HNum.POS_INF, HNum.NEG_INF, new HStr(""), HNum.NaN ],
          [ new HCoord(12.0, -34.0), new HCoord(0.123, -0.789), new HCoord(84.5, -77.45), new HCoord(-90.0, 180.0) ]
        ]
    );
  });

  test('', () {
    verifyGrid(
        "ver:\"2.0\"\n" +
        "foo\n" +
        "`foo\$20bar`\n" +
        "`foo\\`bar`\n" +
        "`file \\#2`\n" +
        "\"\$15\"\n",
        null,
        [
          "foo", null,
        ],
        [
          [ new HUri("foo\$20bar") ],
          [ new HUri("foo`bar") ],
          [ new HUri("file \\#2") ],
          [ new HStr("\$15") ]
        ]
    );
  });

  test('', () {
    verifyGrid(
        "ver:\"2.0\"\n" +
        "a, b\n" +
        "-3.1kg,4kg\n" +
        "5%,3.2%\n" +
        "5kWh/ft\u00b2,-15kWh/m\u00b2\n" +
        "123e+12kJ/kg_dry,74\u0394\u00b0F\n",
        null,
        [
          "a", null,
          "b", null
        ],
        [
          [ new HNum(-3.1, "kg"), new HNum(4.0, "kg") ],
          [ new HNum(5.0, "%"), new HNum(3.2, "%") ],
          [ new HNum(5.0, "kWh/ft\u00b2"), new HNum(-15.0, "kWh/m\u00b2") ],
          [ new HNum(1.23E14, "kJ/kg_dry"), new HNum(74.0, "\u0394\u00b0F") ],
        ]
    );
  });

  test('', () {
    verifyGrid(
        "ver:\"2.0\"\n" +
        "a, b, c\n" +
        ", 1, 2\n" +
        "3, , 5\n" +
        "6, 7_000,\n" +
        ",,10\n" +
        ",,\n" +
        "14,,\n" +
        "\n",
        null,
        [
          "a", null,
          "b", null,
          "c", null,
        ],
        [
          [ null, new HNum(1.0), new HNum(2.0) ],
          [ new HNum(3.0), null, new HNum(5.0) ],
          [ new HNum(6.0), new HNum(7000.0), null ],
          [ null, null, new HNum(10.0) ],
          [ null, null, null ],
          [ new HNum(14.0), null, null ],
        ]
    );
  });

  test('', () {
    verifyGrid(
        "ver:\"2.0\"\n" +
        "a,b\n" +
        "2010-03-01T23:55:00.013-05:00 GMT+5,2010-03-01T23:55:00.013+10:00 GMT-10\n",
        null,
        [
          "a", null,
          "b", null,
        ],
        [
          [ new HDateTime(new HDate(2010, 3, 1),new HTime(23, 55, 0, 13),new HTimeZone("GMT+5", -18000)), new HDateTime(new HDate(2010, 3, 1),new HTime(23, 55, 0, 13),new HTimeZone("GMT-10", 36000)) ]
        ]
    );
  });

  test('', () {
    verifyGrid(
        "ver:\"2.0\" a: 2009-02-03T04:05:06Z foo b: 2010-02-03T04:05:06Z UTC bar c: 2009-12-03T04:05:06Z London baz\n" +
        "a\n" +
        "3.814697265625E-6\n" +
        "2010-12-18T14:11:30.924Z\n" +
        "2010-12-18T14:11:30.925Z UTC\n" +
        "2010-12-18T14:11:30.925Z London\n" +
        "45\$\n" +
        "33\u00a3\n" +
        "@12cbb08e-0c02ae73\n" +
        "7.15625E-4kWh/ft\u00b2\n",
        new HDictBuilder().add("b", new HDateTime(new HDate(2010, 2, 3),new HTime(4, 5, 6, 0),new HTimeZone("UTC"))).add("baz", HMarker.VAL).add("c", new HDateTime(new HDate(2009, 12, 3),new HTime(4, 5, 6, 0),new HTimeZone("London"))).add("a", new HDateTime(new HDate(2009, 2, 3),new HTime(4, 5, 6, 0),new HTimeZone("UTC"))).add("foo", HMarker.VAL).add("bar", HMarker.VAL).toDict(),
        [
          "a", null,
        ],
        [
          [ new HNum(3.814697265625E-6) ],
          [ new HDateTime(new HDate(2010, 12, 18),new HTime(14, 11, 30, 924),new HTimeZone("UTC")) ],
          [ new HDateTime(new HDate(2010, 12, 18),new HTime(14, 11, 30, 925),new HTimeZone("UTC")) ],
          [ new HDateTime(new HDate(2010, 12, 18),new HTime(14, 11, 30, 925),new HTimeZone("London")) ],
          [ new HNum(45.0, "\$") ],
          [ new HNum(33.0, "\u00a3") ],
          [ new HRef("12cbb08e-0c02ae73", null) ],
          [ new HNum(7.15625E-4, "kWh/ft\u00b2") ],
        ]
    );
  });

}

void verifyGrid(String str, HDict meta, List<Object> cols, List<List<HVal>> rows) {
/*
    System.out.println();
    System.out.println("###############################################");
    System.out.println();
    System.out.println(str);
    */

// normalize nulls
  if (meta == null) meta = HDict.EMPTY;
  for (int i=0; i<cols.length; ++i)
    if (cols[i] == null) cols[i] = HDict.EMPTY;

// read from zinc
  HGrid grid = new HZincReader(str).readGrid();
  verifyGridEq(grid, meta, cols, rows);

// write grid and verify we can parse that too
  String writeStr = HZincWriter.gridToString(grid);
  HGrid writeGrid = new HZincReader(writeStr).readGrid();
  verifyGridEq(writeGrid, meta, cols, rows);
}

void verifyGridEq(HGrid grid, HDict meta, List<Object> cols, List<List<HVal>> rows) {
// meta
  expect(grid.meta, equals(meta));

// cols
  expect(grid.numCols, equals(cols.length/2));
  for (int i=0; i<grid.numCols; ++i) {
    expect(grid.colByIndex(i).name, equals(cols[i*2+0]));
    expect(grid.colByIndex(i).meta, equals(cols[i*2+1]));
  }

// rows
  expect(grid.numRows, equals(rows.length));
  for (int ri=0; ri<rows.length; ++ri) {
    List<HVal> expected = rows[ri];
    HRow actual = grid.row(ri);
    for (int ci=0; ci<expected.length; ++ci) {
      expect(expected[ci], equals(actual.get(grid.colByIndex(ci).name, checked:false)));
    }
  }
}
